/* Udp NTP Client Get the time from a Network Time Protocol (NTP) time server
Demonstrates use of UDP sendPacket and ReceivePacket For more on NTP time servers
and the messages needed to communicate with them, see http://en.wikipedia.org/wiki/Network_Time_Protocol
created 4 Sep 2010 by Michael Margolis modified 9 Apr 2012 by Tom Igoe updated for the ESP8266 12 Apr 2015
by Ivan Grokhotkov This code is in the public domain. 

CH_PD, RESET, GPIO0, GPIO2 подтягивают резисторами 10k к VCC модуля esp8266
*/

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <TM1637.h>
#include <Ticker.h>
#include <WiFiManager.h>

Ticker flipper;
TM1637 tm1637(12, 14);              // CLK , DIO (D6, D5)

#define GMT 2                       // GMT shift
#define brightness 0                // value:0-7


byte hour, minute, second;
boolean point;

unsigned int localPort = 2390;      // local port to listen for UDP packets

/* Don't hardwire the IP address or we won't get the benefits of the pool. *  Lookup the IP address for the host name instead */
//IPAddress timeServer(129, 6, 15, 28); // time.nist.gov NTP server
IPAddress timeServerIP; // time.nist.gov NTP server address
const char* ntpServerName = "time.nist.gov";

const int NTP_PACKET_SIZE = 48; // NTP time stamp is in the first 48 bytes of the message

byte packetBuffer[ NTP_PACKET_SIZE]; //buffer to hold incoming and outgoing packets

// A UDP instance to let us send and receive packets over UDP
WiFiUDP udp;

void setup()
{
  //Serial.begin(115200); 
   
  tm1637.init();                      //tm1637 display (4 digits 7 segments)
  tm1637.set(brightness);

  flipper.attach(1, flip);          /// прерывание 1 сек
  
  //Serial.println();

  
  WiFiManager wifiManager;
  wifiManager.autoConnect("AutoConnectAP");
  
 
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    //Serial.print(".");
    WiFi.hostname("watch0button");
  }
  
  udp.begin(localPort);

  oldloop();        /// Synchronize the time when switching on 
}


void loop(){
  
  if (second ==72000){            
    flipper.detach();           
    delay(1500);                
    oldloop();                  
    flipper.attach(1, flip);    
  }
  
   int8_t TimeDisp[4];          // sending everything to the screen 
   tm1637.point(point);         
   
   TimeDisp[0] = hour / 10;
   TimeDisp[1] = hour % 10;
   TimeDisp[2] = minute  / 10;
   TimeDisp[3] = minute  % 10;  
   tm1637.display(TimeDisp);
}

void oldloop()
{
  //get a random server from the pool
  WiFi.hostByName(ntpServerName, timeServerIP); 

  sendNTPpacket(timeServerIP); // send an NTP packet to a time server
  // wait to see if a reply is available
  delay(1000);
  
  int cb = udp.parsePacket();
  if (!cb) {
    //Serial.println("no packet yet");
  }
  else {
    //Serial.print("packet received, length=");
    //Serial.println(cb);
    // We've received a packet, read the data from it
    udp.read(packetBuffer, NTP_PACKET_SIZE); // read the packet into the buffer

    //the timestamp starts at byte 40 of the received packet and is four bytes,
    // or two words, long. First, esxtract the two words:

    unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
    unsigned long lowWord = word(packetBuffer[42], packetBuffer[43]);
    // combine the four bytes (two words) into a long integer
    // this is NTP time (seconds since Jan 1 1900):
    unsigned long secsSince1900 = highWord << 16 | lowWord;
   
    // now convert NTP time into everyday time
    // Unix time starts on Jan 1 1970. In seconds, that's 2208988800:
    const unsigned long seventyYears = 2208988800UL;
    // subtract seventy years:
    unsigned long epoch = secsSince1900 - seventyYears;
    
    //adjusting the time zone and synchronization 
    epoch = epoch + GMT * 3600;     
    
    hour = (epoch  % 86400L) / 3600;
    minute = (epoch  % 3600) / 60;
    second = epoch % 60;
   }
  // wait ten seconds before asking for the time agai
}

// send an NTP request to the time server at the given address
unsigned long sendNTPpacket(IPAddress& address)
{
  //Serial.println("sending NTP packet...");
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49;
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;

  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  udp.beginPacket(address, 123); //NTP requests are to port 123
  udp.write(packetBuffer, NTP_PACKET_SIZE);
  udp.endPacket();
}


void flip(){ 
  point = !point;
  second++; 
  if (second > 59){
    second = 0;
    minute++;      
  }
  if (minute > 59){
    minute = 0;
    hour++;  
  }
  if (hour > 23){
    hour = 0;
  } 

}
